@@ -1,31 +1,25 @@
 from __future__ import annotations
 
-import ast
-import contextlib
-import functools
-import importlib
+import inspect
 import os
 import re
-import sys
 import warnings
 from pathlib import Path
-from typing import Any, Generator
+from typing import Callable
 
+from pdm.backend._vendor.packaging.version import Version
 from pdm.backend.exceptions import ConfigError, PDMWarning, ValidationError
 from pdm.backend.hooks.base import Context
-from pdm.backend.hooks.version.scm import get_version_from_scm
+from pdm.backend.hooks.version.scm import SCMVersion as SCMVersion
+from pdm.backend.hooks.version.scm import (
+    default_version_formatter,
+    get_version_from_scm,
+)
+from pdm.backend.utils import evaluate_module_attribute
 
-_attr_regex = re.compile(r"([\w.]+)\s*:\s*([\w.]+)\s*(\([^)]+\))?")
-
-
-@contextlib.contextmanager
-def patch_sys_path(path: str | Path) -> Generator[None, None, None]:
-    old_path = sys.path[:]
-    sys.path.insert(0, str(path))
-    try:
-        yield
-    finally:
-        sys.path[:] = old_path
+_fallback_version_warning = (
+    "Can't get a valid version from {}, use fallback_version instead."
+)
 
 
 class DynamicVersionBuildHook:
@@ -66,12 +60,12 @@ class DynamicVersionBuildHook:
             )
             return
         options = {k: v for k, v in version_config.items() if k != "source"}
-        metadata["version"] = getattr(self, f"resolve_version_from_{source}")(
-            context, **options
+        metadata["version"] = str(
+            getattr(self, f"resolve_version_from_{source}")(context, **options)
         )
         metadata["dynamic"].remove("version")
 
-    def resolve_version_from_file(self, context: Context, path: str) -> str:
+    def resolve_version_from_file(self, context: Context, path: str) -> Version:
         """Resolve version from a file."""
         version_source = context.root / path
         with open(version_source, encoding="utf-8") as fp:
@@ -83,7 +77,7 @@ class DynamicVersionBuildHook:
                 f"Couldn't find version in file {version_source!r}, "
                 "it should appear as `__version__ = 'a.b.c'`.",
             )
-        return match.group(1)
+        return Version(match.group(1))
 
     def resolve_version_from_scm(
         self,
@@ -91,14 +85,56 @@ class DynamicVersionBuildHook:
         write_to: str | None = None,
         write_template: str = "{}\n",
         tag_regex: str | None = None,
-    ) -> str:
-        if "PDM_BUILD_SCM_VERSION" in os.environ:
+        tag_filter: str | None = None,
+        version_format: str | None = None,
+        fallback_version: str | None = None,
+    ) -> Version:
+        if os.environ.get("PDM_BUILD_SCM_VERSION"):
+            source = "environment variable"
             version = os.environ["PDM_BUILD_SCM_VERSION"]
         else:
-            version = get_version_from_scm(context.root, tag_regex=tag_regex)
-
+            source = "scm"
+            version_formatter: (
+                Callable[[SCMVersion, Context], str] | Callable[[SCMVersion], str]
+            )
+            if version_format is not None:
+                version_formatter, _ = evaluate_module_attribute(
+                    version_format, context.root
+                )
+            else:
+                version_formatter = default_version_formatter
+            scm_version = get_version_from_scm(
+                context.root, tag_regex=tag_regex, tag_filter=tag_filter
+            )
+            if scm_version is None:
+                if fallback_version is not None:
+                    warnings.warn(_fallback_version_warning.format(source), PDMWarning)
+                    version = fallback_version
+                else:
+                    raise ConfigError(
+                        "Cannot find the version from SCM or SCM isn't detected. \n"
+                        "You can still specify the version via environment variable "
+                        "`PDM_BUILD_SCM_VERSION`, or specify `fallback_version` config."
+                    )
+            else:
+                params = inspect.signature(version_formatter).parameters
+                if len(params) > 1:
+                    version = version_formatter(scm_version, context)  # type: ignore[call-arg]
+                else:
+                    version = version_formatter(scm_version)  # type: ignore[call-arg]
+        try:
+            parsed_version = Version(version)
+        except ValueError:
+            if fallback_version is not None:
+                warnings.warn(_fallback_version_warning.format(source), PDMWarning)
+                return Version(fallback_version)
+            raise ConfigError(
+                f"Invalid version {version}, it must comply with PEP 440. \n"
+                "You can still specify the version via environment variable "
+                "`PDM_BUILD_SCM_VERSION`, or specify `fallback_version` config."
+            )
         self._write_version(context, version, write_to, write_template)
-        return version
+        return parsed_version
 
     def _write_version(
         self,
@@ -128,22 +164,18 @@ class DynamicVersionBuildHook:
         getter: str,
         write_to: str | None = None,
         write_template: str = "{}\n",
-    ) -> str:
-        matched = _attr_regex.match(getter)
-        if matched is None:
-            raise ConfigError(
-                "Invalid version getter, must be in the format of "
-                "`module:attribute`."
-            )
-        with patch_sys_path(context.root):
-            module = importlib.import_module(matched.group(1))
-            attrs = matched.group(2).split(".")
-            obj: Any = functools.reduce(getattr, attrs, module)
-            args_group = matched.group(3)
-            if args_group:
-                args = ast.literal_eval(args_group)
+        fallback_version: str | None = None,
+    ) -> Version:
+        version_getter, args = evaluate_module_attribute(getter, context.root)
+        version = version_getter(*args)
+        if version is None:
+            if fallback_version is not None:
+                warnings.warn(_fallback_version_warning.format("call"), PDMWarning)
+                return Version(fallback_version)
             else:
-                args = ()
-            version = obj(*args)
+                raise ConfigError(
+                    "Cannot get version from the call, you can specify fallback version via `fallback_version` config."
+                )
+        parsed_version = Version(version)
         self._write_version(context, version, write_to, write_template)
-        return version
+        return parsed_version
